package geniusweb.protocol.tournament;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

import com.fasterxml.jackson.annotation.JsonAutoDetect;
import com.fasterxml.jackson.annotation.JsonAutoDetect.Visibility;
import com.fasterxml.jackson.annotation.JsonCreator;
import com.fasterxml.jackson.annotation.JsonTypeInfo;
import com.fasterxml.jackson.annotation.JsonValue;

import geniusweb.references.PartyWithParameters;

/**
 * A team is a group of parties, each with their own parameters. Each will be
 * coupled to a {@link ProfileList} in the generation phase of the protocol
 *
 */
@JsonAutoDetect(fieldVisibility = Visibility.ANY, getterVisibility = Visibility.NONE, setterVisibility = Visibility.NONE, isGetterVisibility = Visibility.NONE)
@JsonTypeInfo(use = JsonTypeInfo.Id.NAME, include = JsonTypeInfo.As.WRAPPER_OBJECT)
public class Team {
	@JsonValue
	private final List<PartyWithParameters> parties; // ensure immutability

	/**
	 * 
	 * @param ps a list of {@link PartyWithParameters}. Each party is always
	 *           associated with the given parameters. The order of them is
	 *           important as it will be matched with the order in the
	 *           {@link ProfileList}.
	 */
	@JsonCreator
	public Team(List<PartyWithParameters> ps) {
		this.parties = new ArrayList<PartyWithParameters>(ps);
	}

	/**
	 * 
	 * @return list of all team {@link PartyWithParameters}.
	 */
	public List<PartyWithParameters> getParties() {
		return Collections.unmodifiableList(parties);
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((parties == null) ? 0 : parties.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		Team other = (Team) obj;
		if (parties == null) {
			if (other.parties != null)
				return false;
		} else if (!parties.equals(other.parties))
			return false;
		return true;
	}

	public String toString() {
		return parties.toString();
	}
}
